@@ -17,7 +17,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-frNN <- function(x, eps, sort = TRUE, search = "kdtree", bucketSize = 10,
+frNN <- function(x, eps, query = NULL, sort = TRUE, search = "kdtree", bucketSize = 10,
   splitRule = "suggest", approx = 0) {
 
   if(is.null(eps) || is.na(eps) || eps < 0) stop("eps needs to be >=0.")
@@ -48,6 +48,8 @@ frNN <- function(x, eps, sort = TRUE, search = "kdtree", bucketSize = 10,
   ### get kNN from a dist object in R
   if(is(x, "dist")) {
 
+    if(!is.null(query)) stop("query can only be used if x contains the data.")
+
     if(any(is.na(x))) stop("data/distances cannot contain NAs for frNN (with kd-tree)!")
 
     x <- as.matrix(x)
@@ -68,8 +70,10 @@ frNN <- function(x, eps, sort = TRUE, search = "kdtree", bucketSize = 10,
     )
     names(d) <- rownames(x)
 
-    return(structure(list(dist = d, id = id, eps = eps, sort = TRUE),
-      class = c("frNN", "NN")))
+    ret <- structure(list(dist = d, id = id, eps = eps, sort = TRUE),
+      class = c("frNN", "NN"))
+
+    return(ret)
   }
 
   ## make sure x is numeric
@@ -78,30 +82,39 @@ frNN <- function(x, eps, sort = TRUE, search = "kdtree", bucketSize = 10,
   if(storage.mode(x) == "integer") storage.mode(x) <- "double"
   if(storage.mode(x) != "double") stop("x has to be a numeric matrix.")
 
+  if(!is.null(query)) {
+    query <- as.matrix(query)
+    if(storage.mode(query) == "integer") storage.mode(query) <- "double"
+    if(storage.mode(query) != "double") stop("query has to be NULL or a numeric matrix.")
+    if(ncol(x) != ncol(query)) stop("x and query need to have the same number of columns!")
+  }
+
   splitRule <- pmatch(toupper(splitRule), .ANNsplitRule)-1L
   if(is.na(splitRule)) stop("Unknown splitRule!")
 
   if(any(is.na(x))) stop("data/distances cannot contain NAs for frNN (with kd-tree)!")
 
-  ret <- frNN_int(as.matrix(x), as.double(eps),
-    as.integer(search), as.integer(bucketSize),
-    as.integer(splitRule), as.double(approx))
-
-  if(sort) {
-    o <- lapply(1:length(ret$dist), FUN =
-        function(i) order(ret$dist[[i]], ret$id[[i]], decreasing=FALSE))
-    ret$dist <- lapply(1:length(o), FUN = function(p) ret$dist[[p]][o[[p]]])
-    ret$id <- lapply(1:length(o), FUN = function(p) ret$id[[p]][o[[p]]])
+  ## returns NO self matches
+  if(!is.null(query)) {
+    ret <- frNN_query_int(as.matrix(x), as.matrix(query), as.double(eps),
+      as.integer(search), as.integer(bucketSize),
+      as.integer(splitRule), as.double(approx))
+    names(ret$dist) <- rownames(query)
+    names(ret$id) <- rownames(query)
+  }else{
+    ret <- frNN_int(as.matrix(x), as.double(eps),
+      as.integer(search), as.integer(bucketSize),
+      as.integer(splitRule), as.double(approx))
+    names(ret$dist) <- rownames(x)
+    names(ret$id) <- rownames(x)
   }
 
-  names(ret$dist) <- rownames(x)
-  names(ret$id) <- rownames(x)
-
   ret$eps <- eps
-  ret$sort <- sort
-
+  ret$sort <- FALSE
   class(ret) <- c("frNN", "NN")
 
+  if(sort) ret <- sort.frNN(ret)
+
   ret
 }
 
@@ -109,11 +122,17 @@ sort.frNN <- function(x, decreasing = FALSE, ...) {
   if(!is.null(x$sort) && x$sort) return(x)
   if(is.null(x$dist)) stop("Unable to sort. Distances are missing.")
 
+  ## FIXME: This is slow do this in C++
+  n <- names(x$id)
+
   o <- lapply(1:length(x$dist), FUN =
-      function(i) order(x$dist[[i]], x$id[[i]], decreasing=FALSE))
+      function(i) order(x$dist[[i]], x$id[[i]], decreasing=decreasing))
   x$dist <- lapply(1:length(o), FUN = function(p) x$dist[[p]][o[[p]]])
   x$id <- lapply(1:length(o), FUN = function(p) x$id[[p]][o[[p]]])
 
+  names(x$dist) <- n
+  names(x$id) <- n
+
   x$sort <- TRUE
 
   x
